@@ -75,7 +75,7 @@ psf_get_chunk_iterator (SF_PRIVATE * psf, const char * marker_str)
 		hash = marker_len > 4 ? hash_of_str (marker_str) : u.marker ;
 
 		memcpy (psf->iterator->id, marker_str, marker_len) ;
-		psf->iterator->id_size = marker_len ;
+		psf->iterator->id_size = (unsigned) marker_len ;
 		psf->iterator->hash = hash ;
 		}
 
@@ -112,6 +112,9 @@ psf_store_read_chunk (READ_CHUNKS * pchk, const READ_CHUNK * rchunk)
 	{	pchk->used = 0 ;
 		pchk->count = 20 ;
 		pchk->chunks = calloc (pchk->count, sizeof (READ_CHUNK)) ;
+		if (!pchk->chunks)
+		{	return SFE_MALLOC_FAILED ;
+			} ;
 		}
 	else if (pchk->used > pchk->count)
 		return SFE_INTERNAL ;
@@ -208,7 +211,7 @@ psf_store_read_chunk_str (READ_CHUNKS * pchk, const char * marker_str, sf_count_
 	rchunk.offset = offset ;
 	rchunk.len = len ;
 
-	rchunk.id_size = marker_len > 64 ? 64 : marker_len ;
+	rchunk.id_size = marker_len > 64 ? 64 : (unsigned) marker_len ;
 	memcpy (rchunk.id, marker_str, rchunk.id_size) ;
 
 	return psf_store_read_chunk (pchk, &rchunk) ;
@@ -219,6 +222,7 @@ psf_save_write_chunk (WRITE_CHUNKS * pchk, const SF_CHUNK_INFO * chunk_info)
 {	union
 	{	uint32_t marker ;
 		char str [5] ;
+		/* Update snprintf() format string below when changing this */
 	} u ;
 	uint32_t len ;
 
@@ -226,6 +230,9 @@ psf_save_write_chunk (WRITE_CHUNKS * pchk, const SF_CHUNK_INFO * chunk_info)
 	{	pchk->used = 0 ;
 		pchk->count = 20 ;
 		pchk->chunks = calloc (pchk->count, sizeof (WRITE_CHUNK)) ;
+		if (!pchk->chunks)
+		{	return SFE_MALLOC_FAILED ;
+			} ;
 		}
 	else if (pchk->used >= pchk->count)
 	{	WRITE_CHUNK * old_ptr = pchk->chunks ;
@@ -241,7 +248,7 @@ psf_save_write_chunk (WRITE_CHUNKS * pchk, const SF_CHUNK_INFO * chunk_info)
 	len = chunk_info->datalen ;
 	while (len & 3) len ++ ;
 
-	snprintf (u.str, sizeof (u.str), "%s", chunk_info->id) ;
+	snprintf (u.str, sizeof (u.str), "%.4s", chunk_info->id) ;
 
 	pchk->chunks [pchk->used].hash = strlen (chunk_info->id) > 4 ? hash_of_str (chunk_info->id) : u.marker ;
 	pchk->chunks [pchk->used].mark32 = u.marker ;
