@@ -103,8 +103,8 @@ log_putchar (SF_PRIVATE *psf, char ch)
 void
 psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 {	va_list		ap ;
-	uint32_t	u ;
-	int			d, tens, shift, width, width_specifier, left_align, slen ;
+	uint32_t	u, tens ;
+	int			d, shift, width, width_specifier, left_align, slen, precision ;
 	char		c, *strptr, istr [5], lead_char, sign_char ;
 
 	va_start (ap, format) ;
@@ -153,6 +153,12 @@ psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 		while ((c = *format++) && isdigit (c))
 			width_specifier = width_specifier * 10 + (c - '0') ;
 
+		precision = 0 ;
+		if (c == '.')
+		{	while ((c = *format++) && isdigit (c))
+				precision = precision * 10 + (c - '0') ;
+			} ;
+
 		switch (c)
 		{	case 0 : /* NULL character. */
 					va_end (ap) ;
@@ -162,12 +168,15 @@ psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 					strptr = va_arg (ap, char *) ;
 					if (strptr == NULL)
 						break ;
-					slen = strlen (strptr) ;
+					if (precision > 0)
+						slen = strnlen (strptr, precision) ;
+					else
+						slen = strlen (strptr) ;
 					width_specifier = width_specifier >= slen ? width_specifier - slen : 0 ;
 					if (left_align == SF_FALSE)
 						while (width_specifier -- > 0)
 							log_putchar (psf, ' ') ;
-					while (*strptr)
+					while (slen--)
 						log_putchar (psf, *strptr++) ;
 					while (width_specifier -- > 0)
 						log_putchar (psf, ' ') ;
@@ -177,15 +186,19 @@ psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 					d = va_arg (ap, int) ;
 
 					if (d < 0)
-					{	d = -d ;
-						sign_char = '-' ;
+					{	sign_char = '-' ;
 						if (lead_char != '0' && left_align == SF_FALSE)
 							width_specifier -- ;
-						} ;
+
+						u = - ((unsigned) d) ;
+						}
+					else
+					{	u = (unsigned) d ;
+						}
 
 					tens = 1 ;
 					width = 1 ;
-					while (d / tens >= 10)
+					while (u / tens >= 10)
 					{	tens *= 10 ;
 						width ++ ;
 						} ;
@@ -215,8 +228,8 @@ psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 							log_putchar (psf, lead_char) ;
 
 					while (tens > 0)
-					{	log_putchar (psf, '0' + d / tens) ;
-						d %= tens ;
+					{	log_putchar (psf, '0' + u / tens) ;
+						u %= tens ;
 						tens /= 10 ;
 						} ;
 
@@ -225,7 +238,8 @@ psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 					break ;
 
 			case 'D': /* sf_count_t */
-					{	sf_count_t		D, Tens ;
+					{	sf_count_t	D ;
+						uint64_t	U, Tens ;
 
 						D = va_arg (ap, sf_count_t) ;
 
@@ -235,13 +249,19 @@ psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 							log_putchar (psf, '0') ;
 							break ;
 							}
-						if (D < 0)
-						{	log_putchar (psf, '-') ;
-							D = -D ;
-							} ;
+						else
+						{	if (D < 0)
+							{	log_putchar (psf, '-') ;
+								U = -((uint64_t) D) ;
+								}
+							else
+							{	U = (uint64_t) D;
+								}
+							}
+
 						Tens = 1 ;
 						width = 1 ;
-						while (D / Tens >= 10)
+						while (U / Tens >= 10)
 						{	Tens *= 10 ;
 							width ++ ;
 							} ;
@@ -252,8 +272,8 @@ psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 							} ;
 
 						while (Tens > 0)
-						{	log_putchar (psf, '0' + D / Tens) ;
-							D %= Tens ;
+						{	log_putchar (psf, '0' + U / Tens) ;
+							U %= Tens ;
 							Tens /= 10 ;
 							} ;
 						} ;
@@ -355,7 +375,7 @@ psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 					strptr = istr ;
 					while (*strptr)
 					{	c = *strptr++ ;
-						log_putchar (psf, c) ;
+						log_putchar (psf, psf_isprint (c) ? c : '.') ;
 						} ;
 					break ;
 
@@ -520,36 +540,30 @@ header_put_le_int (SF_PRIVATE *psf, int x)
 	psf->header.ptr [psf->header.indx++] = (x >> 24) ;
 } /* header_put_le_int */
 
-#if (SIZEOF_SF_COUNT_T == 8)
-
 static inline void
 header_put_be_8byte (SF_PRIVATE *psf, sf_count_t x)
 {	psf->header.ptr [psf->header.indx++] = (x >> 56) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 48) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 40) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 32) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 24) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 16) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 8) ;
-	psf->header.ptr [psf->header.indx++] = x ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 48) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 40) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 32) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 24) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 16) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 8) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) x ;
 } /* header_put_be_8byte */
 
 static inline void
 header_put_le_8byte (SF_PRIVATE *psf, sf_count_t x)
-{	psf->header.ptr [psf->header.indx++] = x ;
-	psf->header.ptr [psf->header.indx++] = (x >> 8) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 16) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 24) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 32) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 40) ;
-	psf->header.ptr [psf->header.indx++] = (x >> 48) ;
+{	psf->header.ptr [psf->header.indx++] = (unsigned char) x ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 8) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 16) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 24) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 32) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 40) ;
+	psf->header.ptr [psf->header.indx++] = (unsigned char) (x >> 48) ;
 	psf->header.ptr [psf->header.indx++] = (x >> 56) ;
 } /* header_put_le_8byte */
 
-#else
-#error "SIZEOF_SF_COUNT_T != 8"
-#endif
-
 int
 psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 {	va_list	argptr ;
@@ -571,7 +585,7 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 	while ((c = *format++))
 	{
 		if (psf->header.indx + 16 >= psf->header.len && psf_bump_header_allocation (psf, 16))
-			return count ;
+			break ;
 
 		switch (c)
 		{	case ' ' : /* Do nothing. Just used to space out format string. */
@@ -687,7 +701,7 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					size = strlen (strptr) + 1 ;
 
 					if (psf->header.indx + 4 + (sf_count_t) size + (sf_count_t) (size & 1) > psf->header.len && psf_bump_header_allocation (psf, 4 + size + (size & 1)))
-						return count ;
+						break ;
 
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						header_put_be_int (psf, size + (size & 1)) ;
@@ -708,7 +722,7 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					strptr = va_arg (argptr, char *) ;
 					size = strlen (strptr) ;
 					if (psf->header.indx + 4 + (sf_count_t) size + (sf_count_t) (size & 1) > psf->header.len && psf_bump_header_allocation (psf, 4 + size + (size & 1)))
-						return count ;
+						break ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						header_put_be_int (psf, size) ;
 					else
@@ -728,7 +742,7 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					size = (size > 254) ? 254 : size ;
 
 					if (psf->header.indx + 1 + (sf_count_t) size > psf->header.len && psf_bump_header_allocation (psf, 1 + size))
-						return count ;
+						break ;
 
 					header_put_byte (psf, size) ;
 					memcpy (&(psf->header.ptr [psf->header.indx]), strptr, size) ;
@@ -741,7 +755,7 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					size	= va_arg (argptr, size_t) ;
 
 					if (psf->header.indx + (sf_count_t) size > psf->header.len && psf_bump_header_allocation (psf, size))
-						return count ;
+						break ;
 
 					memcpy (&(psf->header.ptr [psf->header.indx]), bindata, size) ;
 					psf->header.indx += size ;
@@ -752,7 +766,7 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					size = va_arg (argptr, size_t) ;
 
 					if (psf->header.indx + (sf_count_t) size > psf->header.len && psf_bump_header_allocation (psf, size))
-						return count ;
+						break ;
 
 					count += size ;
 					while (size)
@@ -773,7 +787,7 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					size = va_arg (argptr, size_t) ;
 
 					if (psf->header.indx + (sf_count_t) size > psf->header.len && psf_bump_header_allocation (psf, size))
-						return count ;
+						break ;
 
 					psf->header.indx += size ;
 					count += size ;
@@ -783,7 +797,7 @@ psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 					size = va_arg (argptr, size_t) ;
 
 					if ((sf_count_t) size >= psf->header.len && psf_bump_header_allocation (psf, size))
-						return count ;
+						break ;
 
 					psf->header.indx = size ;
 					break ;
@@ -877,7 +891,7 @@ header_seek (SF_PRIVATE *psf, sf_count_t position, int whence)
 				psf_bump_header_allocation (psf, position) ;
 			if (position > psf->header.len)
 			{	/* Too much header to cache so just seek instead. */
-				psf->header.indx = psf->header.end ;
+				psf->header.indx = psf->header.end = 0 ;
 				psf_fseek (psf, position, whence) ;
 				return ;
 				} ;
@@ -905,8 +919,22 @@ header_seek (SF_PRIVATE *psf, sf_count_t position, int whence)
 
 			if (psf->header.indx + position > psf->header.len)
 			{	/* Need to jump this without caching it. */
+				position -= (psf->header.end - psf->header.indx) ;
 				psf->header.indx = psf->header.end ;
-				psf_fseek (psf, position, SEEK_CUR) ;
+				if (psf->is_pipe)
+				{
+					/* seeking is not supported on pipe input, so we read instead */
+					size_t skip = position ;
+					while (skip)
+					{	char junk [16 * 1024] ;
+						size_t to_skip = SF_MIN (skip, sizeof (junk)) ;
+						psf_fread (junk, 1, to_skip, psf) ;
+						skip -= to_skip ;
+						}
+					}
+				else
+				{	psf_fseek (psf, position, SEEK_CUR) ;
+					}
 				break ;
 				} ;
 
@@ -954,7 +982,7 @@ int
 psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 {	va_list			argptr ;
 	sf_count_t		*countptr, countdata ;
-	unsigned char	*ucptr, sixteen_bytes [16] ;
+	unsigned char	*ucptr, sixteen_bytes [16] = { 0 } ;
 	unsigned int 	*intptr, intdata ;
 	unsigned short	*shortptr ;
 	char			*charptr ;
@@ -971,7 +999,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 	while ((c = *format++))
 	{
 		if (psf->header.indx + 16 >= psf->header.len && psf_bump_header_allocation (psf, 16))
-			return count ;
+			break ;
 
 		switch (c)
 		{	case 'e' : /* All conversions are now from LE to host. */
@@ -1098,7 +1126,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 					memset (charptr, 0, count) ;
 
 					if (psf->header.indx + count >= psf->header.len && psf_bump_header_allocation (psf, count))
-						return 0 ;
+						break ;
 
 					byte_count += header_gets (psf, charptr, count) ;
 					break ;
@@ -1127,6 +1155,10 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 					byte_count += count ;
 					break ;
 
+			case '!' : /* Clear buffer, forcing re-read. */
+					psf->header.end = psf->header.indx = 0 ;
+					break ;
+
 			default :
 				psf_log_printf (psf, "*** Invalid format specifier `%c'\n", c) ;
 				psf->error = SFE_INTERNAL ;
@@ -1221,6 +1253,37 @@ psf_log_SF_INFO (SF_PRIVATE *psf)
 /*========================================================================================
 */
 
+int
+psf_isprint (int ch)
+{	return (ch >= ' ' && ch <= '~') ;
+} /* psf_isprint */
+
+void
+psf_strlcat (char *dest, size_t n, const char *src)
+{	strncat (dest, src, n - strlen (dest) - 1) ;
+	dest [n - 1] = 0 ;
+} /* psf_strlcat */
+
+void
+psf_strlcpy (char *dest, size_t n, const char *src)
+{	strncpy (dest, src, n - 1) ;
+	dest [n - 1] = 0 ;
+} /* psf_strlcpy */
+
+/*========================================================================================
+*/
+
+void *
+psf_memdup (const void *src, size_t n)
+{	if (src == NULL)
+		return NULL ;
+
+	void * mem = calloc (1, n & 3 ? n + 4 - (n & 3) : n) ;
+	if (mem != NULL)
+		memcpy (mem, src, n) ;
+	return mem ;
+} /* psf_memdup */
+
 void*
 psf_memset (void *s, int c, sf_count_t len)
 {	char	*ptr ;
@@ -1246,7 +1309,11 @@ psf_memset (void *s, int c, sf_count_t len)
 ** bodgy something up instead.
 */
 
+#ifdef _MSC_VER
 typedef SF_CUES_VAR (0) SF_CUES_0 ;
+#else
+typedef SF_CUES_VAR () SF_CUES_0 ;
+#endif
 
 /* calculate size of SF_CUES struct given number of cues */
 #define SF_CUES_VAR_SIZE(count)	(sizeof (SF_CUES_0) + count * sizeof (SF_CUE_POINT))
@@ -1257,8 +1324,9 @@ typedef SF_CUES_VAR (0) SF_CUES_0 ;
 SF_CUES *
 psf_cues_alloc (uint32_t cue_count)
 {	SF_CUES *pcues = calloc (1, SF_CUES_VAR_SIZE (cue_count)) ;
-
-	pcues->cue_count = cue_count ;
+	if (pcues)
+	{	pcues->cue_count = cue_count ;
+		} ;
 	return pcues ;
 } /* psf_cues_alloc */
 
@@ -1527,6 +1595,7 @@ str_of_major_format (int format)
 		CASE_NAME (SF_FORMAT_CAF) ;
 		CASE_NAME (SF_FORMAT_WVE) ;
 		CASE_NAME (SF_FORMAT_OGG) ;
+		CASE_NAME (SF_FORMAT_MPEG) ;
 		default :
 			break ;
 		} ;
@@ -1563,6 +1632,9 @@ str_of_minor_format (int format)
 		CASE_NAME (SF_FORMAT_DPCM_8) ;
 		CASE_NAME (SF_FORMAT_DPCM_16) ;
 		CASE_NAME (SF_FORMAT_VORBIS) ;
+		CASE_NAME (SF_FORMAT_MPEG_LAYER_I) ;
+		CASE_NAME (SF_FORMAT_MPEG_LAYER_II) ;
+		CASE_NAME (SF_FORMAT_MPEG_LAYER_III) ;
 		default :
 			break ;
 		} ;
@@ -1606,8 +1678,8 @@ psf_f2s_array (const float *src, short *dest, int count, int normalize)
 {	float 			normfact ;
 
 	normfact = normalize ? (1.0 * 0x7FFF) : 1.0 ;
-	while (--count >= 0)
-		dest [count] = psf_lrintf (src [count] * normfact) ;
+	for (int i = 0 ; i < count ; i++)
+		dest [i] = psf_lrintf (src [i] * normfact) ;
 
 	return ;
 } /* psf_f2s_array */
@@ -1618,18 +1690,18 @@ psf_f2s_clip_array (const float *src, short *dest, int count, int normalize)
 
 	normfact = normalize ? (1.0 * 0x8000) : 1.0 ;
 
-	while (--count >= 0)
-	{	scaled_value = src [count] * normfact ;
-		if (CPU_CLIPS_POSITIVE == 0 && scaled_value >= (1.0 * 0x7FFF))
-		{	dest [count] = 0x7FFF ;
+	for (int i = 0 ; i < count ; i++)
+	{	scaled_value = src [i] * normfact ;
+		if (scaled_value >= (1.0 * 0x7FFF))
+		{	dest [i] = 0x7FFF ;
 			continue ;
 			} ;
-		if (CPU_CLIPS_NEGATIVE == 0 && scaled_value <= (-8.0 * 0x1000))
-		{	dest [count] = 0x8000 ;
+		if (scaled_value <= (-8.0 * 0x1000))
+		{	dest [i] = -0x7FFF - 1 ;
 			continue ;
 			} ;
 
-		dest [count] = psf_lrintf (scaled_value) ;
+		dest [i] = psf_lrintf (scaled_value) ;
 		} ;
 
 	return ;
@@ -1640,8 +1712,8 @@ psf_d2s_array (const double *src, short *dest, int count, int normalize)
 {	double 			normfact ;
 
 	normfact = normalize ? (1.0 * 0x7FFF) : 1.0 ;
-	while (--count >= 0)
-		dest [count] = psf_lrint (src [count] * normfact) ;
+	for (int i = 0 ; i < count ; i++)
+		dest [i] = psf_lrint (src [i] * normfact) ;
 
 	return ;
 } /* psf_f2s_array */
@@ -1652,18 +1724,18 @@ psf_d2s_clip_array (const double *src, short *dest, int count, int normalize)
 
 	normfact = normalize ? (1.0 * 0x8000) : 1.0 ;
 
-	while (--count >= 0)
-	{	scaled_value = src [count] * normfact ;
-		if (CPU_CLIPS_POSITIVE == 0 && scaled_value >= (1.0 * 0x7FFF))
-		{	dest [count] = 0x7FFF ;
+	for (int i = 0 ; i < count ; i++)
+	{	scaled_value = src [i] * normfact ;
+		if (scaled_value >= (1.0 * 0x7FFF))
+		{	dest [i] = 0x7FFF ;
 			continue ;
 			} ;
-		if (CPU_CLIPS_NEGATIVE == 0 && scaled_value <= (-8.0 * 0x1000))
-		{	dest [count] = 0x8000 ;
+		if (scaled_value <= (-8.0 * 0x1000))
+		{	dest [i] = -0x7FFF - 1 ;
 			continue ;
 			} ;
 
-		dest [count] = psf_lrint (scaled_value) ;
+		dest [i] = psf_lrint (scaled_value) ;
 		} ;
 
 	return ;
@@ -1675,8 +1747,8 @@ psf_f2i_array (const float *src, int *dest, int count, int normalize)
 {	float 			normfact ;
 
 	normfact = normalize ? (1.0 * 0x7FFFFFFF) : 1.0 ;
-	while (--count >= 0)
-		dest [count] = psf_lrintf (src [count] * normfact) ;
+	for (int i = 0 ; i < count ; i++)
+		dest [i] = psf_lrintf (src [i] * normfact) ;
 
 	return ;
 } /* psf_f2i_array */
@@ -1687,18 +1759,22 @@ psf_f2i_clip_array (const float *src, int *dest, int count, int normalize)
 
 	normfact = normalize ? (8.0 * 0x10000000) : 1.0 ;
 
-	while (--count >= 0)
-	{	scaled_value = src [count] * normfact ;
-		if (CPU_CLIPS_POSITIVE == 0 && scaled_value >= (1.0 * 0x7FFFFFFF))
-		{	dest [count] = 0x7FFFFFFF ;
+	for (int i = 0 ; i < count ; i++)
+	{	scaled_value = src [i] * normfact ;
+#if CPU_CLIPS_POSITIVE == 0
+		if (scaled_value >= (1.0 * 0x7FFFFFFF))
+		{	dest [i] = 0x7FFFFFFF ;
 			continue ;
 			} ;
-		if (CPU_CLIPS_NEGATIVE == 0 && scaled_value <= (-8.0 * 0x10000000))
-		{	dest [count] = 0x80000000 ;
+#endif
+#if CPU_CLIPS_NEGATIVE == 0
+		if (scaled_value <= (-8.0 * 0x10000000))
+		{	dest [i] = 0x80000000 ;
 			continue ;
 			} ;
+#endif
 
-		dest [count] = psf_lrintf (scaled_value) ;
+		dest [i] = psf_lrintf (scaled_value) ;
 		} ;
 
 	return ;
@@ -1709,8 +1785,8 @@ psf_d2i_array (const double *src, int *dest, int count, int normalize)
 {	double 			normfact ;
 
 	normfact = normalize ? (1.0 * 0x7FFFFFFF) : 1.0 ;
-	while (--count >= 0)
-		dest [count] = psf_lrint (src [count] * normfact) ;
+	for (int i = 0 ; i < count ; i++)
+		dest [i] = psf_lrint (src [i] * normfact) ;
 
 	return ;
 } /* psf_f2i_array */
@@ -1721,18 +1797,22 @@ psf_d2i_clip_array (const double *src, int *dest, int count, int normalize)
 
 	normfact = normalize ? (8.0 * 0x10000000) : 1.0 ;
 
-	while (--count >= 0)
-	{	scaled_value = src [count] * normfact ;
-		if (CPU_CLIPS_POSITIVE == 0 && scaled_value >= (1.0 * 0x7FFFFFFF))
-		{	dest [count] = 0x7FFFFFFF ;
+	for (int i = 0 ; i < count ; i++)
+	{	scaled_value = src [i] * normfact ;
+#if CPU_CLIPS_POSITIVE == 0
+		if (scaled_value >= (1.0 * 0x7FFFFFFF))
+		{	dest [i] = 0x7FFFFFFF ;
 			continue ;
 			} ;
-		if (CPU_CLIPS_NEGATIVE == 0 && scaled_value <= (-8.0 * 0x10000000))
-		{	dest [count] = 0x80000000 ;
+#endif
+#if CPU_CLIPS_NEGATIVE == 0
+		if (scaled_value <= (-8.0 * 0x10000000))
+		{	dest [i] = 0x80000000 ;
 			continue ;
 			} ;
+#endif
 
-		dest [count] = psf_lrint (scaled_value) ;
+		dest [i] = psf_lrint (scaled_value) ;
 		} ;
 
 	return ;
