@@ -76,6 +76,34 @@
 
 static void psf_log_syserr (SF_PRIVATE *psf, int error) ;
 
+int
+psf_copy_filename (SF_PRIVATE *psf, const char *path)
+{	const char *ccptr ;
+	char *cptr ;
+
+	if (strlen (path) > 1 && strlen (path) - 1 >= sizeof (psf->file.path))
+	{	psf->error = SFE_FILENAME_TOO_LONG ;
+		return psf->error ;
+		} ;
+
+	snprintf (psf->file.path, sizeof (psf->file.path), "%s", path) ;
+	if ((ccptr = strrchr (path, '/')) || (ccptr = strrchr (path, '\\')))
+		ccptr ++ ;
+	else
+		ccptr = path ;
+
+	snprintf (psf->file.name, sizeof (psf->file.name), "%s", ccptr) ;
+
+	/* Now grab the directory. */
+	snprintf (psf->file.dir, sizeof (psf->file.dir), "%s", path) ;
+	if ((cptr = strrchr (psf->file.dir, '/')) || (cptr = strrchr (psf->file.dir, '\\')))
+		cptr [1] = 0 ;
+	else
+		psf->file.dir [0] = 0 ;
+
+	return 0 ;
+} /* psf_copy_filename */
+
 #if (USE_WINDOWS_API == 0)
 
 /*------------------------------------------------------------------------------
@@ -132,9 +160,9 @@ psf_open_rsrc (SF_PRIVATE *psf)
 		return 0 ;
 
 	/* Test for MacOSX style resource fork on HPFS or HPFS+ filesystems. */
-	count = snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s/..namedfork/rsrc", psf->file.path.c) ;
+	count = snprintf (psf->rsrc.path, sizeof (psf->rsrc.path), "%s/..namedfork/rsrc", psf->file.path) ;
 	psf->error = SFE_NO_ERROR ;
-	if (count < sizeof (psf->rsrc.path.c))
+	if (count < sizeof (psf->rsrc.path))
 	{	if ((psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
 		{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
 			if (psf->rsrclength > 0 || (psf->rsrc.mode & SFM_WRITE))
@@ -153,9 +181,9 @@ psf_open_rsrc (SF_PRIVATE *psf)
 	** Now try for a resource fork stored as a separate file in the same
 	** directory, but preceded with a dot underscore.
 	*/
-	count = snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s._%s", psf->file.dir.c, psf->file.name.c) ;
+	count = snprintf (psf->rsrc.path, sizeof (psf->rsrc.path), "%s._%s", psf->file.dir, psf->file.name) ;
 	psf->error = SFE_NO_ERROR ;
-	if (count < sizeof (psf->rsrc.path.c) && (psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
+	if (count < sizeof (psf->rsrc.path) && (psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
 	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
 		return SFE_NO_ERROR ;
 		} ;
@@ -164,9 +192,9 @@ psf_open_rsrc (SF_PRIVATE *psf)
 	** Now try for a resource fork stored in a separate file in the
 	** .AppleDouble/ directory.
 	*/
-	count = snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s.AppleDouble/%s", psf->file.dir.c, psf->file.name.c) ;
+	count = snprintf (psf->rsrc.path, sizeof (psf->rsrc.path), "%s.AppleDouble/%s", psf->file.dir, psf->file.name) ;
 	psf->error = SFE_NO_ERROR ;
-	if (count < sizeof (psf->rsrc.path.c))
+	if (count < sizeof (psf->rsrc.path))
 	{	if ((psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
 		{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
 			return SFE_NO_ERROR ;
@@ -477,7 +505,7 @@ psf_is_pipe (SF_PRIVATE *psf)
 static sf_count_t
 psf_get_filelen_fd (int fd)
 {
-#if (SIZEOF_OFF_T == 4 && SIZEOF_SF_COUNT_T == 8 && HAVE_FSTAT64)
+#if (SIZEOF_OFF_T == 4 && HAVE_FSTAT64)
 	struct stat64 statbuf ;
 
 	if (fstat64 (fd, &statbuf) == -1)
@@ -572,9 +600,9 @@ psf_open_fd (PSF_FILE * pfile)
 		} ;
 
 	if (mode == 0)
-		fd = open (pfile->path.c, oflag) ;
+		fd = open (pfile->path, oflag) ;
 	else
-		fd = open (pfile->path.c, oflag, mode) ;
+		fd = open (pfile->path, oflag, mode) ;
 
 	return fd ;
 } /* psf_open_fd */
@@ -602,7 +630,7 @@ psf_fsync (SF_PRIVATE *psf)
 #endif
 } /* psf_fsync */
 
-#elif	USE_WINDOWS_API
+#else
 
 /* Win32 file i/o functions implemented using native Win32 API */
 
@@ -620,7 +648,7 @@ psf_fopen (SF_PRIVATE *psf)
 	psf->error = 0 ;
 	psf->file.handle = psf_open_handle (&psf->file) ;
 
-	if (psf->file.handle == NULL)
+	if (psf->file.handle == INVALID_HANDLE_VALUE)
 		psf_log_syserr (psf, GetLastError ()) ;
 
 	return psf->error ;
@@ -634,14 +662,14 @@ psf_fclose (SF_PRIVATE *psf)
 		return 0 ;
 
 	if (psf->file.do_not_close_descriptor)
-	{	psf->file.handle = NULL ;
+	{	psf->file.handle = INVALID_HANDLE_VALUE ;
 		return 0 ;
 		} ;
 
 	if ((retval = psf_close_handle (psf->file.handle)) == -1)
 		psf_log_syserr (psf, GetLastError ()) ;
 
-	psf->file.handle = NULL ;
+	psf->file.handle = INVALID_HANDLE_VALUE ;
 
 	return retval ;
 } /* psf_fclose */
@@ -649,13 +677,13 @@ psf_fclose (SF_PRIVATE *psf)
 /* USE_WINDOWS_API */ int
 psf_open_rsrc (SF_PRIVATE *psf)
 {
-	if (psf->rsrc.handle != NULL)
+	if (psf->rsrc.handle != INVALID_HANDLE_VALUE)
 		return 0 ;
 
 	/* Test for MacOSX style resource fork on HPFS or HPFS+ filesystems. */
-	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s/rsrc", psf->file.path.c) ;
+	snprintf (psf->rsrc.path, sizeof (psf->rsrc.path), "%s/rsrc", psf->file.path) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->rsrc.handle = psf_open_handle (&psf->rsrc)) != NULL)
+	if ((psf->rsrc.handle = psf_open_handle (&psf->rsrc)) != INVALID_HANDLE_VALUE)
 	{	psf->rsrclength = psf_get_filelen_handle (psf->rsrc.handle) ;
 		return SFE_NO_ERROR ;
 		} ;
@@ -664,9 +692,9 @@ psf_open_rsrc (SF_PRIVATE *psf)
 	** Now try for a resource fork stored as a separate file in the same
 	** directory, but preceded with a dot underscore.
 	*/
-	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s._%s", psf->file.dir.c, psf->file.name.c) ;
+	snprintf (psf->rsrc.path, sizeof (psf->rsrc.path), "%s._%s", psf->file.dir, psf->file.name) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->rsrc.handle = psf_open_handle (&psf->rsrc)) != NULL)
+	if ((psf->rsrc.handle = psf_open_handle (&psf->rsrc)) != INVALID_HANDLE_VALUE)
 	{	psf->rsrclength = psf_get_filelen_handle (psf->rsrc.handle) ;
 		return SFE_NO_ERROR ;
 		} ;
@@ -675,19 +703,17 @@ psf_open_rsrc (SF_PRIVATE *psf)
 	** Now try for a resource fork stored in a separate file in the
 	** .AppleDouble/ directory.
 	*/
-	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s.AppleDouble/%s", psf->file.dir.c, psf->file.name.c) ;
+	snprintf (psf->rsrc.path, sizeof (psf->rsrc.path), "%s.AppleDouble/%s", psf->file.dir, psf->file.name) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->rsrc.handle = psf_open_handle (&psf->rsrc)) != NULL)
+	if ((psf->rsrc.handle = psf_open_handle (&psf->rsrc)) != INVALID_HANDLE_VALUE)
 	{	psf->rsrclength = psf_get_filelen_handle (psf->rsrc.handle) ;
 		return SFE_NO_ERROR ;
 		} ;
 
 	/* No resource file found. */
-	if (psf->rsrc.handle == NULL)
+	if (psf->rsrc.handle == INVALID_HANDLE_VALUE)
 		psf_log_syserr (psf, GetLastError ()) ;
 
-	psf->rsrc.handle = NULL ;
-
 	return psf->error ;
 } /* psf_open_rsrc */
 
@@ -738,9 +764,9 @@ psf_get_filelen (SF_PRIVATE *psf)
 
 /* USE_WINDOWS_API */ void
 psf_init_files (SF_PRIVATE *psf)
-{	psf->file.handle = NULL ;
-	psf->rsrc.handle = NULL ;
-	psf->file.hsaved = NULL ;
+{	psf->file.handle = INVALID_HANDLE_VALUE ;
+	psf->rsrc.handle = INVALID_HANDLE_VALUE ;
+	psf->file.hsaved = INVALID_HANDLE_VALUE ;
 } /* psf_init_files */
 
 /* USE_WINDOWS_API */ void
@@ -764,6 +790,7 @@ psf_open_handle (PSF_FILE * pfile)
 	DWORD dwShareMode ;
 	DWORD dwCreationDistribution ;
 	HANDLE handle ;
+	LPWSTR pwszPath = NULL ;
 
 	switch (pfile->mode)
 	{	case SFM_READ :
@@ -785,50 +812,40 @@ psf_open_handle (PSF_FILE * pfile)
 				break ;
 
 		default :
-				return NULL ;
+				return INVALID_HANDLE_VALUE ;
+		} ;
+
+	int nResult = MultiByteToWideChar (CP_UTF8, 0, pfile->path, -1, NULL, 0) ;
+	pwszPath = malloc (nResult * sizeof (WCHAR)) ;
+	if (!pwszPath)
+		return INVALID_HANDLE_VALUE ;
+	
+	int nResult2 = MultiByteToWideChar (CP_UTF8, 0, pfile->path, -1, pwszPath, nResult) ;
+	if (nResult != nResult2)
+	{	free (pwszPath) ;
+		return INVALID_HANDLE_VALUE ;
 		} ;
 
 #if defined (WINAPI_FAMILY_PARTITION) && !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP | WINAPI_PARTITION_SYSTEM)
-	if (!pfile->use_wchar)
-		return NULL ;
-
 	CREATEFILE2_EXTENDED_PARAMETERS cfParams = { 0 } ;
 	cfParams.dwSize = sizeof (CREATEFILE2_EXTENDED_PARAMETERS) ;
 	cfParams.dwFileAttributes = FILE_ATTRIBUTE_NORMAL ;
 
-	handle = CreateFile2 (pfile->path.wc, dwDesiredAccess, dwShareMode, dwCreationDistribution, &cfParams) ;
-
-	if (handle == INVALID_HANDLE_VALUE)
-		return NULL ;
-
-	return handle ;
+	handle = CreateFile2 (pwszPath, dwDesiredAccess, dwShareMode, dwCreationDistribution, &cfParams) ;
 #else
-	if (pfile->use_wchar)
-		handle = CreateFileW (
-					pfile->path.wc,				/* pointer to name of the file */
-					dwDesiredAccess,			/* access (read-write) mode */
-					dwShareMode,				/* share mode */
-					0,							/* pointer to security attributes */
-					dwCreationDistribution,		/* how to create */
-					FILE_ATTRIBUTE_NORMAL,		/* file attributes (could use FILE_FLAG_SEQUENTIAL_SCAN) */
-					NULL						/* handle to file with attributes to copy */
-					) ;
-	else
-		handle = CreateFileA (
-					pfile->path.c,				/* pointer to name of the file */
-					dwDesiredAccess,			/* access (read-write) mode */
-					dwShareMode,				/* share mode */
-					0,							/* pointer to security attributes */
-					dwCreationDistribution,		/* how to create */
-					FILE_ATTRIBUTE_NORMAL,		/* file attributes (could use FILE_FLAG_SEQUENTIAL_SCAN) */
-					NULL						/* handle to file with attributes to copy */
-					) ;
-
-	if (handle == INVALID_HANDLE_VALUE)
-		return NULL ;
+	handle = CreateFileW (
+				pwszPath,					/* pointer to name of the file */
+				dwDesiredAccess,			/* access (read-write) mode */
+				dwShareMode,				/* share mode */
+				0,							/* pointer to security attributes */
+				dwCreationDistribution,		/* how to create */
+				FILE_ATTRIBUTE_NORMAL,		/* file attributes (could use FILE_FLAG_SEQUENTIAL_SCAN) */
+				NULL						/* handle to file with attributes to copy */
+				) ;
+#endif
+	free (pwszPath) ;
 
 	return handle ;
-#endif
 } /* psf_open_handle */
 
 /* USE_WINDOWS_API */ static void
@@ -860,14 +877,14 @@ psf_log_syserr (SF_PRIVATE *psf, int error)
 /* USE_WINDOWS_API */ int
 psf_close_rsrc (SF_PRIVATE *psf)
 {	psf_close_handle (psf->rsrc.handle) ;
-	psf->rsrc.handle = NULL ;
+	psf->rsrc.handle = INVALID_HANDLE_VALUE ;
 	return 0 ;
 } /* psf_close_rsrc */
 
 
 /* USE_WINDOWS_API */ int
 psf_set_stdio (SF_PRIVATE *psf)
-{	HANDLE	handle = NULL ;
+{	HANDLE	handle = INVALID_HANDLE_VALUE ;
 	int	error = 0 ;
 
 	switch (psf->file.mode)
@@ -909,9 +926,7 @@ psf_set_file (SF_PRIVATE *psf, int fd)
 
 /* USE_WINDOWS_API */ int
 psf_file_valid (SF_PRIVATE *psf)
-{	if (psf->file.handle == NULL)
-		return SF_FALSE ;
-	if (psf->file.handle == INVALID_HANDLE_VALUE)
+{	if (psf->file.handle == INVALID_HANDLE_VALUE)
 		return SF_FALSE ;
 	return SF_TRUE ;
 } /* psf_set_file */
@@ -1073,7 +1088,7 @@ psf_ftell (SF_PRIVATE *psf)
 
 /* USE_WINDOWS_API */ static int
 psf_close_handle (HANDLE handle)
-{	if (handle == NULL)
+{	if (handle == INVALID_HANDLE_VALUE)
 		return 0 ;
 
 	if (CloseHandle (handle) == 0)
@@ -1187,386 +1202,5 @@ psf_ftruncate (SF_PRIVATE *psf, sf_count_t len)
 	return retval ;
 } /* psf_ftruncate */
 
-
-#else
-/* Win32 file i/o functions implemented using Unix-style file i/o API */
-
-/* Win32 has a 64 file offset seek function:
-**
-**		__int64 _lseeki64 (int handle, __int64 offset, int origin) ;
-**
-** It also has a 64 bit fstat function:
-**
-**		int fstati64 (int, struct _stati64) ;
-**
-** but the fscking thing doesn't work!!!!! The file size parameter returned
-** by this function is only valid up until more data is written at the end of
-** the file. That makes this function completely 100% useless.
-*/
-
-#include <io.h>
-#include <direct.h>
-
-/* Win32 */ int
-psf_fopen (SF_PRIVATE *psf, const char *pathname, int open_mode)
-{	int oflag, mode ;
-
-	switch (open_mode)
-	{	case SFM_READ :
-				oflag = O_RDONLY | O_BINARY ;
-				mode = 0 ;
-				break ;
-
-		case SFM_WRITE :
-				oflag = O_WRONLY | O_CREAT | O_TRUNC | O_BINARY ;
-				mode = S_IRUSR | S_IWUSR | S_IRGRP ;
-				break ;
-
-		case SFM_RDWR :
-				oflag = O_RDWR | O_CREAT | O_BINARY ;
-				mode = S_IRUSR | S_IWUSR | S_IRGRP ;
-				break ;
-
-		default :
-				psf->error = SFE_BAD_OPEN_MODE ;
-				return -1 ;
-				break ;
-		} ;
-
-	if (mode == 0)
-		psf->file.filedes = open (pathname, oflag) ;
-	else
-		psf->file.filedes = open (pathname, oflag, mode) ;
-
-	if (psf->file.filedes == -1)
-		psf_log_syserr (psf, errno) ;
-
-	return psf->file.filedes ;
-} /* psf_fopen */
-
-/* Win32 */ sf_count_t
-psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
-{	sf_count_t	new_position ;
-
-	if (psf->virtual_io)
-		return psf->vio.seek (offset, whence, psf->vio_user_data) ;
-
-	switch (whence)
-	{	case SEEK_SET :
-				offset += psf->fileoffset ;
-				break ;
-
-		case SEEK_END :
-				if (psf->file.mode == SFM_WRITE)
-				{	new_position = _lseeki64 (psf->file.filedes, offset, whence) ;
-
-					if (new_position < 0)
-						psf_log_syserr (psf, errno) ;
-
-					return new_position - psf->fileoffset ;
-					} ;
-
-				/* Transform SEEK_END into a SEEK_SET, ie find the file
-				** length add the requested offset (should be <= 0) to
-				** get the offset wrt the start of file.
-				*/
-				whence = SEEK_SET ;
-				offset = _lseeki64 (psf->file.filedes, 0, SEEK_END) + offset ;
-				break ;
-
-		default :
-				/* No need to do anything about SEEK_CUR. */
-				break ;
-		} ;
-
-	/*
-	** Bypass weird Win32-ism if necessary.
-	** _lseeki64() returns an "invalid parameter" error if called with the
-	** offset == 0 and whence == SEEK_CUR.
-	*** Use the _telli64() function instead.
-	*/
-	if (offset == 0 && whence == SEEK_CUR)
-		new_position = _telli64 (psf->file.filedes) ;
-	else
-		new_position = _lseeki64 (psf->file.filedes, offset, whence) ;
-
-	if (new_position < 0)
-		psf_log_syserr (psf, errno) ;
-
-	new_position -= psf->fileoffset ;
-
-	return new_position ;
-} /* psf_fseek */
-
-/* Win32 */ sf_count_t
-psf_fread (void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf)
-{	sf_count_t total = 0 ;
-	ssize_t	count ;
-
-	if (psf->virtual_io)
-		return psf->vio.read (ptr, bytes*items, psf->vio_user_data) / bytes ;
-
-	items *= bytes ;
-
-	/* Do this check after the multiplication above. */
-	if (items <= 0)
-		return 0 ;
-
-	while (items > 0)
-	{	/* Break the writes down to a sensible size. */
-		count = (items > SENSIBLE_SIZE) ? SENSIBLE_SIZE : (ssize_t) items ;
-
-		count = read (psf->file.filedes, ((char*) ptr) + total, (size_t) count) ;
-
-		if (count == -1)
-		{	if (errno == EINTR)
-				continue ;
-
-			psf_log_syserr (psf, errno) ;
-			break ;
-			} ;
-
-		if (count == 0)
-			break ;
-
-		total += count ;
-		items -= count ;
-		} ;
-
-	return total / bytes ;
-} /* psf_fread */
-
-/* Win32 */ sf_count_t
-psf_fwrite (const void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf)
-{	sf_count_t total = 0 ;
-	ssize_t	count ;
-
-	if (psf->virtual_io)
-		return psf->vio.write (ptr, bytes*items, psf->vio_user_data) / bytes ;
-
-	items *= bytes ;
-
-	/* Do this check after the multiplication above. */
-	if (items <= 0)
-		return 0 ;
-
-	while (items > 0)
-	{	/* Break the writes down to a sensible size. */
-		count = (items > SENSIBLE_SIZE) ? SENSIBLE_SIZE : items ;
-
-		count = write (psf->file.filedes, ((const char*) ptr) + total, count) ;
-
-		if (count == -1)
-		{	if (errno == EINTR)
-				continue ;
-
-			psf_log_syserr (psf, errno) ;
-			break ;
-			} ;
-
-		if (count == 0)
-			break ;
-
-		total += count ;
-		items -= count ;
-		} ;
-
-	return total / bytes ;
-} /* psf_fwrite */
-
-/* Win32 */ sf_count_t
-psf_ftell (SF_PRIVATE *psf)
-{	sf_count_t pos ;
-
-	if (psf->virtual_io)
-		return psf->vio.tell (psf->vio_user_data) ;
-
-	pos = _telli64 (psf->file.filedes) ;
-
-	if (pos == ((sf_count_t) -1))
-	{	psf_log_syserr (psf, errno) ;
-		return -1 ;
-		} ;
-
-	return pos - psf->fileoffset ;
-} /* psf_ftell */
-
-/* Win32 */ int
-psf_fclose (SF_PRIVATE *psf)
-{	int retval ;
-
-	while ((retval = close (psf->file.filedes)) == -1 && errno == EINTR)
-		/* Do nothing. */ ;
-
-	if (retval == -1)
-		psf_log_syserr (psf, errno) ;
-
-	psf->file.filedes = -1 ;
-
-	return retval ;
-} /* psf_fclose */
-
-/* Win32 */ sf_count_t
-psf_fgets (char *buffer, sf_count_t bufsize, SF_PRIVATE *psf)
-{	sf_count_t	k = 0 ;
-	sf_count_t	count ;
-
-	while (k < bufsize - 1)
-	{	count = read (psf->file.filedes, &(buffer [k]), 1) ;
-
-		if (count == -1)
-		{	if (errno == EINTR)
-				continue ;
-
-			psf_log_syserr (psf, errno) ;
-			break ;
-			} ;
-
-		if (count == 0 || buffer [k++] == '\n')
-			break ;
-		} ;
-
-	buffer [k] = 0 ;
-
-	return k ;
-} /* psf_fgets */
-
-/* Win32 */ int
-psf_is_pipe (SF_PRIVATE *psf)
-{	struct stat statbuf ;
-
-	if (psf->virtual_io)
-		return SF_FALSE ;
-
-	/* Not sure if this works. */
-	if (fstat (psf->file.filedes, &statbuf) == -1)
-	{	psf_log_syserr (psf, errno) ;
-		/* Default to maximum safety. */
-		return SF_TRUE ;
-		} ;
-
-	/* These macros are defined in Win32/unistd.h. */
-	if (S_ISFIFO (statbuf.st_mode) || S_ISSOCK (statbuf.st_mode))
-		return SF_TRUE ;
-
-	return SF_FALSE ;
-} /* psf_checkpipe */
-
-/* Win32 */ sf_count_t
-psf_get_filelen (SF_PRIVATE *psf)
-{
-#if 0
-	/*
-	** Windoze is SOOOOO FUCKED!!!!!!!
-	** This code should work but doesn't. Why?
-	** Code below does work.
-	*/
-	struct _stati64 statbuf ;
-
-	if (_fstati64 (psf->file.filedes, &statbuf))
-	{	psf_log_syserr (psf, errno) ;
-		return (sf_count_t) -1 ;
-		} ;
-
-	return statbuf.st_size ;
-#else
-	sf_count_t current, filelen ;
-
-	if (psf->virtual_io)
-		return psf->vio.get_filelen (psf->vio_user_data) ;
-
-	if ((current = _telli64 (psf->file.filedes)) < 0)
-	{	psf_log_syserr (psf, errno) ;
-		return (sf_count_t) -1 ;
-		} ;
-
-	/*
-	** Lets face it, windoze if FUBAR!!!
-	**
-	** For some reason, I have to call _lseeki64() TWICE to get to the
-	** end of the file.
-	**
-	** This might have been avoided if windows had implemented the POSIX
-	** standard function fsync() but NO, that would have been too easy.
-	**
-	** I am VERY close to saying that windoze will no longer be supported
-	** by libsndfile and changing the license to GPL at the same time.
-	*/
-
-	_lseeki64 (psf->file.filedes, 0, SEEK_END) ;
-
-	if ((filelen = _lseeki64 (psf->file.filedes, 0, SEEK_END)) < 0)
-	{	psf_log_syserr (psf, errno) ;
-		return (sf_count_t) -1 ;
-		} ;
-
-	if (filelen > current)
-		_lseeki64 (psf->file.filedes, current, SEEK_SET) ;
-
-	switch (psf->file.mode)
-	{	case SFM_WRITE :
-			filelen = filelen - psf->fileoffset ;
-			break ;
-
-		case SFM_READ :
-			if (psf->fileoffset > 0 && psf->filelength > 0)
-				filelen = psf->filelength ;
-			break ;
-
-		case SFM_RDWR :
-			/*
-			** Cannot open embedded files SFM_RDWR so we don't need to
-			** subtract psf->fileoffset. We already have the answer we
-			** need.
-			*/
-			break ;
-
-		default :
-			filelen = 0 ;
-		} ;
-
-	return filelen ;
-#endif
-} /* psf_get_filelen */
-
-/* Win32 */ int
-psf_ftruncate (SF_PRIVATE *psf, sf_count_t len)
-{	int retval ;
-
-	/* Returns 0 on success, non-zero on failure. */
-	if (len < 0)
-		return 1 ;
-
-	/* The global village idiots at micorsoft decided to implement
-	** nearly all the required 64 bit file offset functions except
-	** for one, truncate. The fscking morons!
-	**
-	** This is not 64 bit file offset clean. Somone needs to clean
-	** this up.
-	*/
-	if (len > 0x7FFFFFFF)
-		return -1 ;
-
-	retval = chsize (psf->file.filedes, len) ;
-
-	if (retval == -1)
-		psf_log_syserr (psf, errno) ;
-
-	return retval ;
-} /* psf_ftruncate */
-
-
-static void
-psf_log_syserr (SF_PRIVATE *psf, int error)
-{
-	/* Only log an error if no error has been set yet. */
-	if (psf->error == 0)
-	{	psf->error = SFE_SYSTEM ;
-		snprintf (psf->syserr, sizeof (psf->syserr), "System error : %s", strerror (error)) ;
-		} ;
-
-	return ;
-} /* psf_log_syserr */
-
 #endif
 
